from typing import Dict, Any, List, TypedDict, Optional

from filum_utils.clients.common import BaseClient
from filum_utils.clients.log import LogClient, LogType
from filum_utils.config import config
from filum_utils.enums import ObjectType
from filum_utils.types.mini_app import UpdateInstalledMiniApp


class InstalledMiniAppError(TypedDict):
    type: str
    message: Optional[str]
    sub_message: Optional[str]
    data: Any


class InstalledMiniAppsClient(BaseClient):
    def __init__(self):
        super().__init__(
            base_url=config.APPSTORE_BASE_URL,
            username=config.APPSTORE_USERNAME,
            password=config.APPSTORE_PASSWORD
        )

    def get_list(self, mini_app_ids: List[int] = None, page: int = 0, size: int = 100):
        data = self._request(
            method="GET",
            endpoint="/internal/installed-mini-apps",
            params={
                "page": page,
                "size": size,
                "mini_app_ids": mini_app_ids
            }
        )

        return data.get("items") or []


class InstalledMiniAppClient(BaseClient):
    def __init__(self, installed_mini_app_id: int = None, installed_mini_app: Dict[str, Any] = None):
        super().__init__(
            base_url=config.APPSTORE_BASE_URL,
            username=config.APPSTORE_USERNAME,
            password=config.APPSTORE_PASSWORD
        )

        if installed_mini_app:
            self.installed_mini_app = installed_mini_app
        else:
            self.installed_mini_app = self._request(
                method="GET",
                endpoint=f"/internal/installed-mini-apps/{installed_mini_app_id}"
            )

        self.log_client = LogClient()

    def get_data(self, key: str):
        data = self.installed_mini_app.get("data")
        return data and data.get(key)

    def get_service_account_info(self):
        return self.installed_mini_app and self.installed_mini_app.get("service_account_info")

    def update(self, updated_object: UpdateInstalledMiniApp):
        self.installed_mini_app = self._request(
            method="PUT",
            endpoint=f"/internal/installed-mini-apps/{self.installed_mini_app['id']}",
            data=updated_object
        )

    def update_data(self, updated_data: Dict[str, Any]):
        self.installed_mini_app = self._request(
            method="PUT",
            endpoint=f"/internal/installed-mini-apps/{self.installed_mini_app['id']}/data",
            data=updated_data
        )

    def handle_update_failed(
        self,
        error: InstalledMiniAppError,
        trigger_data: Optional[Dict[str, Any]] = None
    ):
        error_message = (
            error.get("message") or "An error occurred when updating installed mini app"
        )
        self._log(
            type=LogType.ERROR,
            code="400",
            title=error_message,
            subtitle=error.get("sub_message"),
            trigger_data=trigger_data,
            error_data=error["data"],
        )

    def _log(
        self,
        type: str,
        code: str,
        title: str,
        subtitle: str,
        trigger_data: Optional[Dict[str, Any]] = None,
        error_data: Optional[Dict[str, Any]] = None
    ):
        self.log_client.create_log(
            object_type=ObjectType.INSTALLED_MINI_APP,
            object_id=self.installed_mini_app["id"],
            type=type,
            code=code,
            title=title,
            subtitle=subtitle,
            trigger_data=trigger_data,
            error_data=error_data,
            member_organization_id=self.installed_mini_app.get("organization_id"),
        )



from typing import TypedDict, Optional, Dict, Any, List


class DataMapping(TypedDict, total=False):
    piped_text: str
    value: str
    value_type: Optional[str]


class TriggerData(TypedDict, total=False):
    segment_id: Optional[str]
    file_name: Optional[str]
    user_limit_per_trigger: Optional[int]
    data_mapping: Optional[List[DataMapping]]


class ZaloTemplateParam(TypedDict, total=False):
    name: str
    require: Optional[bool]  # based on ZNS template parameter name
    type: str
    maxLength: Optional[int]  # based on ZNS template parameter name
    minLength: Optional[int]  # based on ZNS template parameter name
    acceptNull: Optional[bool]  # based on ZNS template parameter name


class SubscriptionData(TypedDict, total=False):
    input_data: Optional[Dict[str, Any]]
    last_current_index: Optional[int]
    last_current_row: Optional[int]
    triggered_source: Optional[str]
    trigger_data: Optional[TriggerData]
    distribution_id: Optional[str]
    zalo_template_id: Optional[str]
    zalo_template_params: Optional[List[ZaloTemplateParam]]
    zalo_template_param_mappings: Optional[Dict[str, str]]

class Subscription(TypedDict, total=False):
    id: str
    data: Optional[SubscriptionData]



import json
from typing import List

from google.cloud import pubsub_v1

from filum_utils.config import config
from filum_utils.types.conversation import PubsubMessageType


class GoogleCloudClient:
    def __init__(self, project_id: str = config.GOOGLE_PROJECT_ID):
        self._project_id = project_id

    def publish_messages(
        self,
        messages: List[PubsubMessageType],
        topic_id: str = config.GOOGLE_PUBSUB_TOPIC_ID,
    ):
        publisher_options = pubsub_v1.types.PublisherOptions()
        publisher = pubsub_v1.PublisherClient(publisher_options=publisher_options)

        topic_path = publisher.topic_path(self._project_id, topic_id)

        for message in messages:
            message = json.dumps(message)
            data = message.encode("utf-8")
            publisher.publish(topic_path, data=data)



from typing import TypedDict, Optional, Dict, Any


class ActionType(TypedDict, total=False):
    id: int
    internal_data: Optional[Dict[str, Any]]


class Action(TypedDict, total=False):
    id: int
    data: Optional[Dict[str, Any]]
    service_account_info: Optional[Any]
    action_type: ActionType



from typing import Dict, Any, TypedDict, Optional, List

from filum_utils.clients.common import BaseClient
from filum_utils.config import config


class ActionError(TypedDict, total=False):
    type: str
    message: Optional[str]
    sub_message: Optional[str]
    data: Dict[str, Any]
    notification_message: Optional[str]


class ActionsClient(BaseClient):
    def __init__(self):
        super().__init__(
            base_url=config.APPSTORE_BASE_URL,
            username=config.APPSTORE_USERNAME,
            password=config.APPSTORE_PASSWORD
        )

    def get_actions(
        self,
        action_type_id: int,
        organization_id: str = None
    ) -> Optional[List[Dict[str, Any]]]:
        return self._request(
            method="GET",
            endpoint="/internal/actions",
            params={
                "action_type_id": action_type_id,
                "organization_id": organization_id
            },
        )


class ActionClient(BaseClient):
    def __init__(self, action_id: int = None, action: Dict[str, Any] = None):
        super().__init__(
            base_url=config.APPSTORE_BASE_URL,
            username=config.APPSTORE_USERNAME,
            password=config.APPSTORE_PASSWORD
        )

        if action:
            self.action = action

        else:
            self.action = self._request(
                method="GET",
                endpoint=f"/source/actions/{action_id}"
            )

    def get_data(self, key):
        data = self.action.get("data")
        return data and data.get(key)

    def update_data(self, updated_data: Dict[str, Any]):
        self.action = self._request(
            method="PUT",
            endpoint=f"/source/actions/{self.action['id']}/data",
            data=updated_data
        )



from typing import TypedDict, Optional, List, Union

from ..enums import PubSubMessageEventNameEnum, MessageSenderTypeEnum, MessageStatusEnum


class MessageUserType(TypedDict, total=False):
    id: str
    name: Optional[str]
    phone: Optional[str]
    email: Optional[str]
    avatar: Optional[str]


class MessageAgentType(TypedDict, total=False):
    id: str
    name: Optional[str]
    email: Optional[str]


class MessageRecordingType(TypedDict, total=False):
    url: Optional[str]
    duration_seconds: Optional[int]


class MessageAttachmentType(TypedDict, total=False):
    internal_url: Optional[str]
    url: str
    size_bytes: Optional[int]
    sizeBytes: Optional[int]
    type: str
    name: Optional[str]
    mime_type: Optional[str]
    mimeType: Optional[str]


class PubsubMessageEventType(TypedDict, total=False):
    timestamp: Optional[str]
    user: MessageUserType
    original_message_id: str
    sender_type: MessageSenderTypeEnum
    original_agent: Optional[MessageAgentType]
    call_center_phone: Optional[str]
    text: Optional[str]
    conversation_link: Optional[str]
    recording: Optional[MessageRecordingType]
    note: Optional[str]
    hangup_cause: Optional[str]
    charge_fee: Optional[float]
    attachments: Optional[List[MessageAttachmentType]]
    campaign_id: Optional[str]
    campaign_name: Optional[str]


class PubsubMessageStatusEventType(TypedDict, total=False):
    timestamp: Optional[str]
    original_message_id: str
    status: MessageStatusEnum


class PubsubMessageType(TypedDict, total=False):
    access_token: str
    organization_id: str
    installed_source_id: int
    source_id: str
    source: str
    event_name: PubSubMessageEventNameEnum
    data: Union[PubsubMessageEventType, PubsubMessageStatusEventType]



from typing import Dict, List, TypedDict, Any, Optional

from filum_utils.config import config
from filum_utils.enums import ObjectType
from .common import BaseClient
from .log import LogClient, StatusCode, LogType
from .notification import NotificationClient, PublisherType, RoutePath, NOTIFICATION_ERROR_MESSAGE_MAPPINGS

VIOLATION_BATCH_SIZE = 2000


class InstalledSource(TypedDict):
    id: int
    access_token: Optional[str]
    data: Dict[str, Any]


class EventPropertyViolation(TypedDict):
    event_property_mapping_id: int
    error_code: str
    origin: str


class EventViolation(TypedDict):
    event_mapping_id: int
    installed_source_id: int
    violation_count: int
    processed_count: int
    event_property_violations: List[EventPropertyViolation]


class SyncResult(TypedDict):
    record_count: int
    error_count: int


class ConnectionError(TypedDict):
    type: str
    message: Optional[str]
    sub_message: Optional[str]
    data: Dict[str, Any]


class ConnectionsClient(BaseClient):
    def __init__(
        self,
        base_url=config.APPSTORE_BASE_URL,
        username=config.APPSTORE_USERNAME,
        password=config.APPSTORE_PASSWORD
    ):
        super().__init__(base_url, username, password)

    def get_connections(
        self,
        data_key: Optional[str] = None,
        data_value: Optional[str] = None,
        source_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        installed_mini_app_id: Optional[str] = None
    ):
        params = {}
        if data_key and data_value:
            params["data_key"] = data_key
            params["data_value"] = data_value

        if source_id:
            params["source_id"] = source_id

        if organization_id:
            params["organization_id"] = organization_id

        if installed_mini_app_id:
            params["installed_mini_app_id"] = installed_mini_app_id

        return self._request(
            method="GET",
            endpoint="/source/installed-sources",
            params=params,
        )


class ConnectionClient(BaseClient):
    def __init__(
        self,
        connection_id: int = None,
        connection=None,
        base_url=config.APPSTORE_BASE_URL,
        username=config.APPSTORE_USERNAME,
        password=config.APPSTORE_PASSWORD
    ):
        super().__init__(base_url, username, password)
        if not connection:
            self.connection = self._request(
                method="GET",
                endpoint=f"/source/installed-sources/{connection_id}",
            )
        else:
            self.connection = connection
        self.log_client = LogClient()
        self.notification_client = NotificationClient()

    def get_access_token(self):
        return self.connection and self.connection.get("access_token")

    def get_webhook_url(self):
        return self.connection and self.connection.get("webhook_endpoint_url")

    def get_service_account_info(self):
        return self.connection and self.connection.get("service_account_info")

    def get_data(self, key: str):
        data = self.connection.get("data")
        return data and data.get(key)

    def update_data(self, updated_data: Dict[str, Any]):
        self._request(
            method="PUT",
            endpoint=f"/source/installed-sources/{self.connection['id']}/data",
            data=updated_data
        )

    def update_status(self, status: str):
        self._request(
            method="PUT",
            endpoint=f"/source/installed-sources/{self.connection['id']}/status",
            data={
                "status": status
            }
        )

    def get_event_mappings(self):
        return self._request(
            method="GET",
            endpoint=f"/source/installed-sources/{self.connection['id']}/event-mappings/all",
        )

    def update_event_mapping_data(self, event_mapping_id, updated_data: Dict[str, Any]):
        self._request(
            method="PUT",
            endpoint=f"/source/event-mappings/{event_mapping_id}/data",
            data=updated_data,
        )

    def sync(self):
        self._request(
            method="POST",
            endpoint=f"/source/installed-sources/{self.connection['id']}/sync",
        )

    def create_event_violations(self, violations: List[EventViolation]):
        if not violations:
            return
        self._request(
            method="POST",
            endpoint=f"/source/installed-sources/{self.connection['id']}/event-violation-batch",
            data=violations,
        )

    def create_event_property_violations(self, violations: List[EventPropertyViolation]):
        if not violations:
            return
        if len(violations) <= VIOLATION_BATCH_SIZE:
            self._request(
                method="POST",
                endpoint=f"/source/event-property-violation-batch",
                data=violations,
            )
            return
        start_index = 0
        while start_index < len(violations):
            end_index = start_index + VIOLATION_BATCH_SIZE
            self._request(
                method="POST",
                endpoint=f"/source/event-property-violation-batch",
                data=violations[start_index:end_index],
            )
            start_index = end_index

    def handle_sync_finished(self, result: SyncResult, notify: bool = False):
        record_count = result["record_count"]
        error_count = result["error_count"]
        if error_count > 0:
            log_type = LogType.WARNING
            status_code = StatusCode.SYNC_WARNING
            title = "Your data has been processed with violations. Please check the Violations tab for details."
            subtitle = f"{record_count} data record(s) processed. {error_count} warning(s)."
        else:
            log_type = LogType.INFO
            status_code = StatusCode.SYNC_SUCCESS
            title = "Your data has been processed successfully."
            subtitle = f"{record_count} data record(s) processed."

        self._log(
            type=log_type,
            code=status_code,
            title=title,
            subtitle=subtitle,
        )

        if notify:
            self._notify(
                title=f"{self.connection['name']} has finished importing data",
                subtitle=subtitle,
            )

    def handle_sync_failed(
        self,
        error: ConnectionError,
        trigger_data: Optional[Dict[str, Any]] = None,
        notify: bool = False
    ):
        error_message = (
            error.get("message") or "An error occurred when processing your data, please contact Filum for support."
        )
        self._log(
            type=LogType.ERROR,
            code=StatusCode.SYNC_ERROR,
            title=error_message,
            subtitle=error.get("sub_message"),
            trigger_data=trigger_data,
            error_data=error["data"],
        )

        if notify:
            self._notify(
                title=f"{self.connection.get('name')} has error",
                subtitle=NOTIFICATION_ERROR_MESSAGE_MAPPINGS[error["type"]],
            )

    def handle_setup_failed(
        self,
        error: ConnectionError,
        trigger_data: Optional[Dict[str, Any]] = None,
        notify: bool = False
    ):
        error_message = (
            error.get("message") or "An error occurred when setup your source, please contact Filum for support."
        )
        self._log(
            type=LogType.ERROR,
            code=StatusCode.SETUP_ERROR,
            title=error_message,
            subtitle=error.get("sub_message"),
            trigger_data=trigger_data,
            error_data=error["data"],
        )

        if notify:
            self._notify(
                title=f"{self.connection.get('name')} has error",
                subtitle=NOTIFICATION_ERROR_MESSAGE_MAPPINGS[error["type"]],
            )

    def _log(
        self,
        type: str,
        code: str,
        title: str,
        subtitle: str,
        trigger_data: Optional[Dict[str, Any]] = None,
        error_data: Optional[Dict[str, Any]] = None
    ):
        self.log_client.create_log(
            object_type=ObjectType.INSTALLED_SOURCE,
            object_id=self.connection["id"],
            type=type,
            code=code,
            title=title,
            subtitle=subtitle,
            trigger_data=trigger_data,
            error_data=error_data,
            member_account_id=self.connection.get("member_account_id"),
            member_organization_id=self.connection.get("organization_id"),
        )

    def _notify(self, title: str, subtitle: str):
        self.notification_client.create_notification(
            publisher_type=PublisherType.DATA_CONNECTION,
            title=title,
            subtitle=subtitle,
            route=self._get_log_route(),
            member_account_id=self.connection.get("member_account_id"),
            member_organization_id=self.connection.get("organization_id"),
        )

    def _get_log_route(self):
        return {
            "path": RoutePath.CONNECTIONS_DETAIL,
            "params": {
                "connectionId": self.connection["id"],
                "tab": "logs"
            }
        }



from typing import TypedDict, Optional, Dict, Any

from filum_utils.types.account import Account


class EngagementCampaign(TypedDict, total=False):
    id: str
    name: str
    account: Account
    data: Optional[Dict[str, Any]]



from typing import Dict, TypedDict

from filum_utils.clients.common import BaseClient
from filum_utils.config import config
from filum_utils.enums import BaseEnum


class PublisherType(BaseEnum):
    DATA_CONNECTION = "Data Connection"
    VoC = "Voice of the Customer"
    AUTOMATED_ACTION = "Automated Action"


class RoutePath(BaseEnum):
    CONNECTIONS_DETAIL = "connectionsDetail"
    AUTOMATED_ACTIONS_DETAIL = "automatedActionsDetail"
    CAMPAIGNS_DETAIL = "listeningCampaignDetails"
    ENGAGEMENT_CAMPAIGNS_DETAIL = "engagementCampaignDetails"


class ErrorType:
    INTERNAL = "internal"
    EXTERNAL = "external"


class NotificationErrorMessage:
    INTERNAL = "Internal system error, please contact Filum for support"
    EXTERNAL = "Partner error, please contact Filum for support"


NOTIFICATION_ERROR_MESSAGE_MAPPINGS = {
    ErrorType.INTERNAL: NotificationErrorMessage.INTERNAL,
    ErrorType.EXTERNAL: NotificationErrorMessage.EXTERNAL,
}


class Route(TypedDict):
    path: str
    params: Dict[str, str]


class NotificationClient(BaseClient):
    def __init__(self):
        super().__init__(
            base_url=config.NOTIFICATION_BASE_URL,
            username=config.NOTIFICATION_USERNAME,
            password=config.NOTIFICATION_PASSWORD
        )

    def create_notification(
        self,
        publisher_type: str,
        title: str,
        subtitle: str,
        route: Route,
        member_account_id: int = None,
        member_organization_id: str = None,
    ):
        notification = {
            "title": title,
            "subtitle": subtitle,
            "publisher_type": publisher_type,
            "route": route,
            "member_account_id": member_account_id,
            "member_organization_id": member_organization_id
        }

        return self._request(
            method="POST",
            endpoint="/internal/notifications",
            data=notification
        )



from typing import TypedDict, Optional, Dict, Any, List


class SurveyLogo(TypedDict, total=False):
    light_mode_url: Optional[str]
    dark_mode_url: Optional[str]


class Survey(TypedDict, total=False):
    id: int
    input_data: Optional[Dict[str, Any]]
    questions: Optional[List[Dict[str, Any]]]
    view_token: Optional[str]
    contact_form_enabled: Optional[bool]
    logo: Optional[SurveyLogo]
    translations: Optional[Dict[str, Any]]
    languages: Optional[List[Dict[str, Any]]]



from abc import ABC, abstractmethod
from typing import Dict, Any, Optional, Callable, List

from filum_utils.clients.action import ActionError
from filum_utils.clients.filum import FilumClient
from filum_utils.clients.log import LogClient, LogType, StatusCode
from filum_utils.clients.notification import (
    NotificationClient,
    NOTIFICATION_ERROR_MESSAGE_MAPPINGS,
)
from filum_utils.clients.subscription import SubscriptionClient
from filum_utils.types.organization import Organization
from filum_utils.types.subscription import Subscription, SubscriptionData, TriggerData

Event = Optional[Dict[str, Any]]
User = Optional[Dict[str, Any]]
Object = Optional[Dict[str, Any]]


class SubscriptionService(ABC):
    def __init__(self, subscription: Subscription, organization: Organization):
        self.subscription_client = SubscriptionClient(subscription, organization)
        self.filum_client = FilumClient()
        self.log_client = LogClient()
        self.notification_client = NotificationClient()

        self.subscription = subscription
        self.organization = organization

    @property
    def subscription_id(self) -> str:
        return self.subscription["id"]

    @property
    def subscription_data(self) -> SubscriptionData:
        return self.subscription.get("data")

    @property
    def trigger_data(self) -> TriggerData:
        return self.subscription_data.get("trigger_data") or {}

    @property
    @abstractmethod
    def parent(self):
        ...

    @property
    @abstractmethod
    def member_account_id(self):
        ...

    @property
    @abstractmethod
    def run_type(self):
        ...

    @property
    @abstractmethod
    def _parent_id(self):
        ...

    @property
    @abstractmethod
    def _parent_name(self):
        ...

    @property
    @abstractmethod
    def _parent_type(self):
        ...

    @property
    @abstractmethod
    def _object_type(self):
        ...

    @property
    @abstractmethod
    def _object_id(self):
        ...

    @property
    @abstractmethod
    def _notification_route(self):
        ...

    @property
    @abstractmethod
    def _notification_publisher_type(self):
        ...

    def update_subscription_data(self, updated_data: Dict[str, Any]):
        self.subscription_client.update_data(updated_data)

    def handle_trigger_successfully(
        self,
        title: str,
        subtitle: Optional[str] = None,
        trigger_data: Optional[Dict[str, Any]] = None,
        notify: bool = False,
    ):
        self._log(
            type=LogType.INFO,
            code=StatusCode.SYNC_SUCCESS,
            title=title,
            subtitle=subtitle,
            trigger_data=trigger_data,
        )

        if notify:
            self._notify(
                publisher_type=f"{self._notification_publisher_type}",
                title=f"{self.parent.get('name')} ran successfully",
                subtitle=subtitle or "",
            )

    def handle_trigger_failed(
        self,
        error: ActionError,
        trigger_data: Optional[Dict[str, Any]] = None,
        notify: bool = False,
    ):
        error_message = (
            error.get("message") or "An error occurred when triggering your action, please contact Filum for support."
        )
        self._log(
            type=LogType.ERROR,
            code=StatusCode.SYNC_ERROR,
            title=error_message,
            subtitle=error.get("sub_message"),
            trigger_data=trigger_data,
            error_data=error["data"],
        )

        if notify:
            self._notify(
                publisher_type=f"{self._notification_publisher_type}",
                title=f"{self.parent.get('name')} failed to distribute",
                subtitle=error.get("notification_message") or NOTIFICATION_ERROR_MESSAGE_MAPPINGS[error["type"]],
            )

    @abstractmethod
    def handle_real_time_trigger(self, process_real_time_fn: Callable, event: [Dict[str, Any]], **kwargs):
        ...

    @abstractmethod
    def handle_segment_manual_trigger(
        self,
        process_segment_manual_fn: Callable,
        properties: List[str],
        required_properties: Optional[List[List[str]]] = None,
        last_current_index: int = 0,
        **kwargs,
    ):
        ...

    @abstractmethod
    def handle_file_manual_trigger(
        self,
        process_file_manual_fn: Callable,
        file_name: str,
        last_current_row: int = 0,
        **kwargs,
    ):
        ...

    @abstractmethod
    def handle_object_manual_trigger(
        self,
        process_object_manual_fn: Callable,
        **kwargs,
    ):
        ...

    def _log(
        self,
        type: str,
        code: str,
        title: str,
        subtitle: str,
        trigger_data: Optional[Dict[str, Any]] = None,
        error_data: Optional[Dict[str, Any]] = None
    ):
        self.log_client.create_log(
            object_type=self._object_type,
            object_id=self._object_id,
            parent_type=self._parent_type,
            parent_id=self.parent["id"],
            type=type,
            code=code,
            title=title,
            subtitle=subtitle,
            trigger_data=trigger_data,
            error_data=error_data,
            member_account_id=self.member_account_id,
            member_organization_id=self.organization.get("id"),
        )

    def _notify(self, publisher_type: str, title: str, subtitle: str):
        self.notification_client.create_notification(
            publisher_type=publisher_type,
            title=title,
            subtitle=subtitle,
            route=self._notification_route,
            member_account_id=self.member_account_id,
            member_organization_id=self.organization.get("id"),
        )



from filum_utils.clients.common import BaseClient
from filum_utils.config import config


class IAMClient(BaseClient):
    def __init__(
        self,
        base_url: str = config.IAM_BASE_URL,
        username: str = config.IAM_USERNAME,
        password: str = config.IAM_PASSWORD
    ):
        super().__init__(base_url, username, password)

    def get_organization(self, organization_id: str):
        return self._request(
            method="GET",
            endpoint=f"/internal/organizations/{organization_id}"
        )



from typing import TypedDict, Optional, Dict, Any

from filum_utils.types.account import Account


class _Context(TypedDict):
    id: str
    type: str


class AutomatedAction(TypedDict, total=False):
    id: int
    name: str
    run_type: str
    account: Account
    context: Optional[_Context]
    data: Optional[Dict[str, Any]]



from typing import TypedDict, Optional

from filum_utils.enums import EngagementMessageStatus


class EngagementMessageUser(TypedDict, total=False):
    phone: Optional[str]
    email: Optional[str]
    name: Optional[str]
    user_id: Optional[str]
    anonymous_id: Optional[str]


class CreateEngagementMessage(TypedDict, total=False):
    id: str
    status: EngagementMessageStatus
    distribution_id: int
    user: EngagementMessageUser
    send_timestamp: str
    send_cost: Optional[float]
    sending_failed_reason: Optional[str]



import json
from typing import Dict, TypedDict, Any, Optional

from filum_utils.clients.common import BaseClient
from filum_utils.config import config


class LogType:
    INFO = "info"
    WARNING = "warning"
    ERROR = "error"


class StatusCode:
    SYNC_SUCCESS = "DC101"
    SYNC_WARNING = "DC200"
    SYNC_ERROR = "DC403"
    SETUP_ERROR = "DC402"


class Route(TypedDict):
    path: str
    params: Dict[str, str]


class LogClient(BaseClient):
    def __init__(self):
        super().__init__(
            base_url=config.NOTIFICATION_BASE_URL,
            username=config.NOTIFICATION_USERNAME,
            password=config.NOTIFICATION_PASSWORD
        )

    def create_log(
        self,
        object_type: str,
        object_id: str,
        type: str,
        code: str,
        title: str,
        subtitle: str,
        parent_type: str = None,
        parent_id: str = None,
        error_data: Optional[Dict[str, Any]] = None,
        trigger_data: Optional[Dict[str, Any]] = None,
        member_account_id: str = None,
        member_organization_id: str = None,
    ):
        data = {}
        if error_data:
            data["error_data"] = error_data

        if trigger_data:
            data["trigger_data"] = trigger_data

        data = self._to_string(data)
        log = {
            "type": type,
            "code": code,
            "title": title,
            "subtitle": subtitle,
            "data": data,
            "object_type": object_type,
            "object_id": object_id,
            "parent_type": parent_type,
            "parent_id": parent_id,
            "member_account_id": member_account_id,
            "member_organization_id": member_organization_id,
        }
        return self._request(
            method="POST",
            endpoint="/internal/logs",
            data=log
        )

    @staticmethod
    def _to_string(default_dict: Dict[str, Any]) -> str:
        return json.dumps(default_dict)



from typing import TypedDict, Dict, Any, Optional


class CreateSurveyResponse(TypedDict, total=False):
    identifier: str
    anonymous_id: str
    send_timestamp: str
    transaction_id: Optional[str]
    user_phone: Optional[str]
    user_email: Optional[str]
    user_name: Optional[str]
    user_id: Optional[str]
    source: Optional[str]
    triggered_source: Optional[str]
    sending_failed_reason: Optional[str]
    distribution_id: Optional[int]
    trigger_data: Optional[Dict[str, Any]]
    send_cost: Optional[float]



from abc import abstractmethod
from typing import Dict, Any, Optional, Callable, List, Union

from filum_utils.clients.notification import PublisherType
from filum_utils.config import config
from filum_utils.enums import BaseStatus, ObjectType
from filum_utils.errors import BaseError, ErrorMessage
from filum_utils.services.file import FileService
from filum_utils.services.subscription import SubscriptionService
from filum_utils.types.action import Action
from filum_utils.types.campaign import Campaign
from filum_utils.types.common import CallableResponse, TriggerFunctionResponse
from filum_utils.types.engageement_campaign import EngagementCampaign
from filum_utils.types.organization import Organization
from filum_utils.types.subscription import Subscription, SubscriptionData

Event = Optional[Dict[str, Any]]
User = Optional[Dict[str, Any]]


class BaseCampaignSubscriptionService(SubscriptionService):
    def __init__(
        self,
        subscription: Subscription,
        action: Action,
        organization: Organization
    ):
        super().__init__(subscription, organization)
        self.action = action

    @property
    def _object_type(self) -> str:
        return ObjectType.ACTION

    @property
    def _object_id(self) -> int:
        return self.action["id"]

    @property
    def _notification_publisher_type(self) -> str:
        return PublisherType.VoC

    @property
    def _user_limit(self) -> int:
        return self.trigger_data.get("user_limit_per_trigger") or 0

    @abstractmethod
    def update_status(self, updated_status: str):
        ...

    @abstractmethod
    def _get_trigger_completed_notification_subtitle(self, channel_name: str, success_count: int) -> str:
        ...

    def handle_real_time_trigger(
        self,
        process_real_time_fn: Callable[
            [Action, Union[Campaign, EngagementCampaign], Organization, Event, SubscriptionData, Any],
            CallableResponse
        ],
        event: [Dict[str, Any]],
        **kwargs,
    ) -> TriggerFunctionResponse:
        result = self._handle_trigger(
            process_real_time_fn,
            event,
            **kwargs
        )

        return {
            "is_finished": True,
            "success_count": result.get("success_count"),
            "error_message": None,
        }

    def handle_segment_manual_trigger(
        self,
        process_segment_manual_fn: Callable[
            [Action, Union[Campaign, EngagementCampaign], Organization, List[User], SubscriptionData, Any],
            CallableResponse
        ],
        properties: List[str],
        last_current_index: int = 0,
        last_success_count: int = 0,
        channel_name: str = None,
        **kwargs,
    ) -> TriggerFunctionResponse:
        self._validate_last_current_index(last_current_index)

        segment_id = self.trigger_data.get("segment_id")
        if not segment_id:
            raise BaseError(
                message=ErrorMessage.MISSING_SEGMENT_ID,
                data={
                    "Campaign ID": self._parent_id,
                }
            )

        users = self.filum_client.get_user_csv_reader(
            properties=properties,
            object_ids=[segment_id],
            object_type="segment",
            organization=self.organization,
            offset=last_current_index,
            limit=config.SEGMENT_RECORD_LIMIT
        )

        return self._handle_manual_trigger(
            process_fn=process_segment_manual_fn,
            users=users,
            object_record_limit=config.SEGMENT_RECORD_LIMIT,
            last_current_index=last_current_index,
            last_success_count=last_success_count,
            channel_name=channel_name,
            **kwargs,
        )

    def handle_file_manual_trigger(
        self,
        process_file_manual_fn: Callable,
        last_current_index: int = 0,
        last_success_count: int = 0,
        channel_name: str = None,
        **kwargs,
    ):
        self._validate_last_current_index(last_current_index)

        file_name = self.trigger_data.get("file_name")
        if not file_name:
            raise BaseError(
                message=ErrorMessage.MISSING_FILE,
                data={
                    "Campaign ID": self._parent_id,
                }
            )

        file_content_bytes = self.filum_client.get_uploaded_file(file_name)
        users = FileService.get_rows(
            file_name,
            file_content_bytes,
            current_index=last_current_index,
            limit=config.FILE_RECORD_LIMIT,
        )

        return self._handle_manual_trigger(
            process_fn=process_file_manual_fn,
            users=users,
            object_record_limit=config.FILE_RECORD_LIMIT,
            last_current_index=last_current_index,
            last_success_count=last_success_count,
            channel_name=channel_name,
            **kwargs,
        )

    def handle_object_manual_trigger(
        self,
        process_object_manual_fn: Callable,
        **kwargs,
    ):
        ...

    def _exceeded_user_limit(self, current_total_users: int) -> bool:
        return current_total_users >= self._user_limit if self._user_limit else False

    def _validate_last_current_index(self, last_current_index: int):
        current_index = self.subscription_data.get("last_current_index") or 0
        if not current_index or current_index == last_current_index:
            return

        raise BaseError(
            message=ErrorMessage.MISMATCH_LAST_CURRENT_INDEX,
            data={
                "Campaign ID": self._parent_id,
                "Current Index": current_index,
                "Last Current Index": last_current_index
            }
        )

    def _handle_manual_trigger(
        self,
        process_fn: Callable,
        users: List[Dict[str, Any]],
        object_record_limit: int,
        last_current_index: int = 0,
        last_success_count: int = 0,
        channel_name: str = None,
        **kwargs
    ):
        result = self._handle_trigger(
            process_fn=process_fn,
            data=users,
            **kwargs,
        )

        success_count = result.get("success_count") or 0
        total_success_count = last_success_count + success_count

        total_users_in_page = len(users) if users else 0
        has_more_users = total_users_in_page >= object_record_limit

        current_index = total_users_in_page + last_current_index
        user_limit_exceeded = self._exceeded_user_limit(current_index)

        stop_running = user_limit_exceeded or not has_more_users
        if stop_running:
            # set campaign completed and stop running
            error_message = self._handle_trigger_function_with_try_except(
                "Update Campaign Status to Completed",
                self._handle_trigger_completed,
                fn_params={
                    "channel_name": channel_name,
                    "success_count": total_success_count
                }
            )
        else:
            # handle running next page
            error_message = self._handle_publish_subscription(
                last_current_index=current_index,
                last_success_count=total_success_count,
            )

        return {
            "is_finished": stop_running,
            "success_count": success_count,
            "error_message": error_message,
        }

    def _handle_trigger(
        self,
        process_fn: Callable,
        data: Any,
        **kwargs,
    ):
        params = {
            "action": self.action,
            "campaign": self.parent,
            "data": data,
            "subscription_data": self.subscription_data,
            "organization": self.organization,
            **kwargs
        }

        return process_fn(**params)

    def _handle_trigger_completed(
        self,
        channel_name: Optional[str],
        success_count: int
    ) -> str:
        update_status_error_message = self._handle_trigger_function_with_try_except(
            "Update Subscription Status",
            self.update_status,
            fn_params={
                "updated_status": BaseStatus.COMPLETED
            }
        )

        update_subscription_data_error_message = self._update_subscription_data_with_try_except({
            "last_current_index": 0
        })

        notify_error_message = ""
        if channel_name:
            subtitle = self._get_trigger_completed_notification_subtitle(
                channel_name, success_count
            )
            notify_error_message = self._handle_trigger_function_with_try_except(
                "Create Notification",
                self._notify,
                fn_params={
                    "publisher_type": f"{self._notification_publisher_type}",
                    "title": f"{self._parent_name} has been distributed successfully to your recipients",
                    "subtitle": subtitle,
                }
            )

        error_messages = {
            update_subscription_data_error_message,
            update_status_error_message,
            notify_error_message,
        }
        return " ".join([error_message for error_message in error_messages if error_message])

    def _handle_publish_subscription(
        self,
        last_current_index: int,
        last_success_count: int
    ):
        update_subscription_data_error_message = self._update_subscription_data_with_try_except({
            "last_current_index": last_current_index
        })
        publish_subscription_error_message = self._handle_trigger_function_with_try_except(
            "Publish Subscription",
            self.subscription_client.publish,
            fn_params={
                "request_data": {
                    "last_current_index": last_current_index,
                    "last_success_count": last_success_count,
                }
            }
        )
        error_message = None
        if update_subscription_data_error_message or publish_subscription_error_message:
            error_message = (
                f"{update_subscription_data_error_message} {publish_subscription_error_message}"
            )

        return error_message

    def _update_subscription_data_with_try_except(self, updated_data: Dict[str, Any]) -> str:
        return self._handle_trigger_function_with_try_except(
            "Update Subscription Data",
            self.update_subscription_data,
            fn_params={
                "updated_data": updated_data
            }
        )

    @staticmethod
    def _handle_trigger_function_with_try_except(
        fn_name: str,
        fn: Callable,
        fn_params: Dict[str, Any] = None
    ) -> str:
        error_message = None
        try:
            fn_params = fn_params if fn_params else {}
            fn(**fn_params)
        except BaseError as e:
            error_message = e.message
        except Exception:
            error_message = ErrorMessage.IMPLEMENTATION_ERROR

        return f"{fn_name}: {error_message}" if error_message else ""



from typing import TypedDict, Optional, Dict, Any

from filum_utils.types.account import Account
from filum_utils.types.survey import Survey


class Campaign(TypedDict, total=False):
    id: str
    name: str
    run_type: Optional[str]
    touchpoint_id: Optional[int]
    feedback_type: Optional[str]
    account: Account
    survey: Survey
    data: Optional[Dict[str, Any]]
    voucher_batch_id: Optional[int]



from typing import TypedDict, Optional, Dict, Any

from filum_utils.types.action import Action
from filum_utils.types.automated_action import AutomatedAction
from filum_utils.types.campaign import Campaign
from filum_utils.types.engageement_campaign import EngagementCampaign
from filum_utils.types.subscription import Subscription


class MessagePayload(TypedDict, total=False):
    organization_id: str
    organization_slug: str
    data: Optional[Dict[str, Any]]
    subscription: Subscription
    action: Action
    campaign: Optional[Campaign]
    engagement_campaign: Optional[EngagementCampaign]
    automated_action: Optional[AutomatedAction]


class CallableResponse(TypedDict):
    success_count: int


class TriggerFunctionResponse(TypedDict):
    is_finished: bool
    success_count: Optional[int]
    error_message: Optional[str]



import logging
import requests
from json import JSONDecodeError
from requests.auth import HTTPBasicAuth
from filum_utils.errors import BaseError


class BaseClient:
    def __init__(self, base_url: str, username: str = None, password: str = None):
        self._base_url = base_url
        self._username = username
        self._password = password

    def _request(self, method, endpoint, data=None, params=None, timeout: float = None):
        response = requests.request(
            method=method,
            url=self._base_url + endpoint,
            json=data,
            params=params,
            auth=HTTPBasicAuth(self._username, self._password),
            timeout=timeout,
        )

        if response.status_code >= 300:
            logging.error(f"{self.__class__} error: {response.status_code} - {endpoint} - {response.content}")
            raise BaseError(f"Implementation Error", data={
                "method": method,
                "endpoint": endpoint,
                "params": params,
                "status_code": response.status_code,
                "content": str(response.content)
            })
        try:
            return response.json()
        except JSONDecodeError:
            return response.content



from typing import Dict, Any

from filum_utils.clients.notification import RoutePath
from filum_utils.enums import ParentType
from filum_utils.services.subscription.base_campaign import BaseCampaignSubscriptionService
from filum_utils.types.action import Action
from filum_utils.types.engageement_campaign import EngagementCampaign
from filum_utils.types.organization import Organization
from filum_utils.types.subscription import Subscription


class EngagementCampaignSubscriptionService(BaseCampaignSubscriptionService):
    def __init__(
        self,
        engagement_campaign: EngagementCampaign,
        subscription: Subscription,
        action: Action,
        organization: Organization
    ):
        super().__init__(subscription, action, organization)
        self.campaign = engagement_campaign

    @property
    def parent(self):
        return self.campaign

    @property
    def member_account_id(self):
        account = self.campaign["account"] or {}
        return account.get("id")

    @property
    def run_type(self) -> str:
        return ""

    @property
    def _parent_id(self) -> str:
        return self.parent["id"]

    @property
    def _parent_name(self) -> str:
        return self.parent["name"]

    @property
    def _parent_type(self) -> str:
        return ParentType.ENGAGEMENT_CAMPAIGN

    @property
    def _notification_route(self) -> Dict[str, Any]:
        return {
            "path": RoutePath.ENGAGEMENT_CAMPAIGNS_DETAIL,
            "params": {
                "campaignId": self._parent_id
            }
        }

    def update_status(self, updated_status: str):
        self.filum_client.update_engagement_campaign_subscription_status(
            organization_id=self.organization["id"],
            campaign_id=self._parent_id,
            distribution_id=self.subscription_data.get("distribution_id"),
            updated_status=updated_status,
        )

    def _get_trigger_completed_notification_subtitle(
        self,
        channel_name: str,
        success_count: int
    ) -> str:
        return f"{success_count} message(s) sent via {channel_name}"



from typing import List

from tenacity import retry, stop_after_attempt, wait_fixed

from filum_utils.clients.common import BaseClient
from filum_utils.config import config
from filum_utils.enums import Organization
from filum_utils.types.engagement_message import CreateEngagementMessage
from filum_utils.types.survey_response import CreateSurveyResponse

_RETRY_PARAMS = {
    "wait": wait_fixed(10),
    "stop": stop_after_attempt(3),
}


class FilumClient(BaseClient):
    def __init__(self):
        super().__init__(
            base_url=config.FILUM_BASE_URL,
            username=config.FILUM_USERNAME,
            password=config.FILUM_PASSWORD,
        )

    def get_automated_action(self, automated_action_id: int):
        return self._request(
            method="GET",
            endpoint=f"/internal/automated-actions/{automated_action_id}"
        )

    def get_campaign(self, campaign_id: str):
        return self._request(
            method="GET",
            endpoint=f"/internal/survey-campaigns/{campaign_id}"
        )

    def get_segment(self, segment_id: str, organization: Organization):
        return self._request(
            method="GET",
            endpoint=f"/internal/segments/{segment_id}",
            params={
                "organization_id": organization["id"],
                "organization_slug": organization["slug"]
            }
        )

    def update_campaign_subscription_status(
        self,
        campaign_id: str,
        subscription_id: str,
        updated_status: str
    ):
        return self._request(
            method="PUT",
            endpoint=f"/internal/survey-campaigns/{campaign_id}/subscription-webhooks",
            data={
                "subscription_id": subscription_id,
                "status": updated_status
            }
        )

    def update_engagement_campaign_subscription_status(
        self,
        organization_id: str,
        campaign_id: str,
        distribution_id: str,
        updated_status: str
    ):
        return self._request(
            method="PUT",
            endpoint=f"/internal/engagement-campaigns/{campaign_id}/distributions/{distribution_id}/status",
            params={
                "organization_id": organization_id
            },
            data={
                "status": updated_status
            }
        )

    def get_user_csv_reader(
        self,
        properties: List[str],
        object_ids: List[str],
        object_type: str,
        organization: Organization,
        limit: int = config.SEGMENT_RECORD_LIMIT,
        offset: int = 0
    ):
        return self._request(
            method="POST",
            endpoint="/internal/users/export",
            data={
                "object_ids": object_ids,
                "object_type": object_type,
                "organization_id": organization["id"],
                "properties": properties,
                "offset": offset,
                "limit": limit
            },
            timeout=120,
        )

    def get_uploaded_file(self, file_name):
        return self._request(
            method="POST",
            endpoint="/internal/download-file",
            data={
                "file_name": file_name,
            }
        )

    @retry(**_RETRY_PARAMS)
    def create_survey_responses(self, campaign_id: str, responses: List[CreateSurveyResponse]):
        return self._request(
            method="POST",
            endpoint=f"/internal/survey-campaigns/{campaign_id}/survey-responses/batch",
            data=responses,
        )

    @retry(**_RETRY_PARAMS)
    def create_engagement_messages(
        self,
        organization_id: str,
        campaign_id: str,
        messages: List[CreateEngagementMessage]
    ):
        return self._request(
            method="POST",
            endpoint=f"/internal/engagement-campaigns/{campaign_id}/messages/batch",
            params={
                "organization_id": organization_id
            },
            data=messages,
        )



from .clients.action import ActionClient
from .clients.analytics import AnalyticsClient
from .clients.connection import ConnectionClient
from .clients.iam import IAMClient
from .clients.mini_app import InstalledMiniAppClient
from .clients.subscription import SubscriptionClient
from .services.subscription.automated_action import AutomatedActionSubscriptionService
from .services.subscription.campaign import CampaignSubscriptionService

__all__ = [
    ActionClient,
    AnalyticsClient,
    ConnectionClient,
    IAMClient,
    InstalledMiniAppClient,
    SubscriptionClient,
    AutomatedActionSubscriptionService,
    CampaignSubscriptionService,
]



from typing import Dict, Any, Optional

from filum_utils.clients.notification import RoutePath
from filum_utils.enums import ParentType
from filum_utils.services.subscription.base_campaign import BaseCampaignSubscriptionService
from filum_utils.types.action import Action
from filum_utils.types.campaign import Campaign
from filum_utils.types.organization import Organization
from filum_utils.types.subscription import Subscription

Event = Optional[Dict[str, Any]]
User = Optional[Dict[str, Any]]


class CampaignSubscriptionService(BaseCampaignSubscriptionService):
    def __init__(
        self,
        campaign: Campaign,
        subscription: Subscription,
        action: Action,
        organization: Organization
    ):
        super().__init__(subscription, action, organization)
        self.campaign = campaign

    @property
    def parent(self):
        return self.campaign

    @property
    def member_account_id(self):
        account = self.campaign["account"] or {}
        return account.get("id")

    @property
    def run_type(self) -> Optional[str]:
        return self.campaign.get("run_type")

    @property
    def _parent_id(self) -> str:
        return self.parent["id"]

    @property
    def _parent_name(self) -> str:
        return self.parent["name"]

    @property
    def _parent_type(self) -> str:
        return ParentType.CAMPAIGN

    @property
    def _notification_route(self) -> Dict[str, Any]:
        return {
            "path": RoutePath.CAMPAIGNS_DETAIL,
            "params": {
                "campaignId": self._parent_id
            }
        }

    def update_status(self, updated_status: str):
        self.filum_client.update_campaign_subscription_status(
            campaign_id=self._parent_id,
            subscription_id=self.subscription_id,
            updated_status=updated_status,
        )

    def _get_trigger_completed_notification_subtitle(
        self,
        channel_name: str,
        success_count: int
    ) -> str:
        return f"{success_count} survey(s) sent via {channel_name}"



from typing import Dict, Any

from tenacity import retry, stop_after_delay, wait_exponential

from filum_utils.clients.common import BaseClient
from filum_utils.config import config
from filum_utils.enums import Organization
from filum_utils.types.subscription import Subscription

_RETRY_PARAMS = {
    "reraise": True,
    "wait": wait_exponential(
        multiplier=2,
    ),
    "stop": stop_after_delay(60),
}


class SubscriptionClient(BaseClient):
    def __init__(self, subscription: Subscription, organization: Organization):
        super().__init__(
            base_url=config.SUBSCRIPTION_BASE_URL,
            username=config.SUBSCRIPTION_USERNAME,
            password=config.SUBSCRIPTION_PASSWORD
        )

        self.subscription = subscription
        self.organization = organization

    @retry(**_RETRY_PARAMS)
    def update_data(self, updated_data: Dict[str, Any]):
        self._request(
            method="PUT",
            endpoint=f"/internal/subscriptions/{self.subscription['id']}",
            params={"organization_id": self.organization["id"]},
            data={"data": updated_data},
        )

    @retry(**_RETRY_PARAMS)
    def publish(self, request_data: Dict[str, Any]):
        self._request(
            method="POST",
            endpoint=f"/internal/subscriptions/{self.subscription.get('id')}/publish",
            data={**request_data}
        )



import os


class Config:
    APPSTORE_BASE_URL = "https://appstore.filum.asia"
    APPSTORE_USERNAME = "appstore@filum.ai"
    APPSTORE_PASSWORD = ""

    NOTIFICATION_BASE_URL = "https://notification.filum.asia"
    NOTIFICATION_USERNAME = "notification@filum.ai"
    NOTIFICATION_PASSWORD = ""

    SUBSCRIPTION_BASE_URL = "https://subscription.filum.asia"
    SUBSCRIPTION_USERNAME = "subscription@filum"
    SUBSCRIPTION_PASSWORD = ""

    FILUM_BASE_URL = "https://api.filum.asia"
    FILUM_USERNAME = "filum_dev"
    FILUM_PASSWORD = ""

    IAM_BASE_URL = "https://iam.filum.asia"
    IAM_USERNAME = "filum_dev"
    IAM_PASSWORD = ""

    EVENT_API_BASE_URL = "https://event.filum.asia"

    SEGMENT_RECORD_LIMIT = 1000
    FILE_RECORD_LIMIT = 1000

    SURVEY_RESPONSE_LIMIT = 1000
    ENGAGEMENT_MESSAGE_LIMIT = 1000

    GOOGLE_PROJECT_ID = ""
    GOOGLE_PUBSUB_TOPIC_ID = ""
    GOOGLE_PUBSUB_API_ENDPOINT = ""

    def __init__(self):
        for attr in dir(self):
            if not attr.isupper():
                continue
            default_value = getattr(self, attr)
            value_type = type(default_value)

            value = os.getenv(attr) or default_value

            if value and isinstance(default_value, list) and isinstance(value, str):
                value = value.split(",")
            elif value_type is bool and isinstance(value, str):
                value = value.lower() in ["true", "yes", "t", "i", "1"]
            else:
                value = value_type(value)

            setattr(self, attr, value)


config = Config()



from typing import Dict, Any, Optional, Callable, List

from filum_utils.clients.notification import RoutePath, PublisherType
from filum_utils.enums import ParentType, ObjectType
from filum_utils.services.subscription import SubscriptionService
from filum_utils.types.action import Action
from filum_utils.types.automated_action import AutomatedAction
from filum_utils.types.common import CallableResponse, TriggerFunctionResponse
from filum_utils.types.organization import Organization
from filum_utils.types.subscription import Subscription, SubscriptionData

Event = Optional[Dict[str, Any]]
Object = Optional[Dict[str, Any]]


class AutomatedActionSubscriptionService(SubscriptionService):
    def __init__(
        self,
        automated_action: AutomatedAction,
        subscription: Subscription,
        action: Action,
        organization: Organization
    ):
        super().__init__(subscription, organization)

        self.automated_action = automated_action
        self.action = action

    @property
    def parent(self):
        return self.automated_action

    @property
    def member_account_id(self):
        account = self.automated_action["account"] or {}
        return account.get("id")

    @property
    def run_type(self) -> Optional[str]:
        return self.automated_action.get("run_type")

    @property
    def _parent_type(self) -> str:
        return ParentType.AUTOMATED_ACTION

    @property
    def _object_type(self) -> str:
        return ObjectType.ACTION

    @property
    def _object_id(self) -> int:
        return self.action["id"]

    @property
    def _notification_route(self) -> Dict[str, Any]:
        return {
            "path": RoutePath.AUTOMATED_ACTIONS_DETAIL,
            "params": {
                "automatedActionId": self.automated_action["id"]
            }
        }

    @property
    def _notification_publisher_type(self) -> str:
        return PublisherType.AUTOMATED_ACTION

    def handle_real_time_trigger(
        self,
        process_real_time_fn: Callable[
            [Action, AutomatedAction, Organization, Event, SubscriptionData, Any],
            CallableResponse
        ],
        event: [Dict[str, Any]],
        **kwargs
    ) -> TriggerFunctionResponse:
        result = self._handle_trigger(
            process_real_time_fn,
            event,
            **kwargs
        )

        return {
            "is_finished": True,
            "success_count": result.get("success_count"),
            "error_message": None
        }

    def handle_segment_manual_trigger(
        self,
        process_segment_manual_fn: Callable,
        properties: List[str],
        required_properties: Optional[List[List[str]]] = None,
        last_current_index: int = 0,
        **kwargs,
    ):
        ...

    def handle_file_manual_trigger(
        self,
        process_file_manual_fn: Callable,
        last_current_row: int = 0,
        **kwargs,
    ):
        ...

    def handle_object_manual_trigger(
        self,
        process_object_manual_fn: Callable[
            [Action, AutomatedAction, Organization, Object, SubscriptionData, Any],
            CallableResponse
        ],
        **kwargs,
    ) -> TriggerFunctionResponse:
        context = self.automated_action.get("context") or {}
        context_id = context.get("id")
        context_type = context.get("type")

        data = {}
        if context_type == Object.SEGMENT:
            data = self.filum_client.get_segment(
                segment_id=context_id,
                organization=self.subscription_client.get_organization()
            )
        elif context_type == Object.CAMPAIGN:
            data = self.filum_client.get_campaign(campaign_id=context_id)

        result = self._handle_trigger(
            process_fn=process_object_manual_fn,
            data=data,
            **kwargs,
        )

        return {
            "is_finished": True,
            "success_count": result.get("success_count"),
            "error_message": None,
        }

    def _handle_trigger(
        self,
        process_fn: Callable,
        data: Any,
        **kwargs,
    ):
        params = {
            "action": self.action,
            "automated_action": self.automated_action,
            "data": data,
            "subscription_data": self.subscription_data,
            "organization": self.organization,
            **kwargs
        }

        return process_fn(**params)



from typing import TypedDict, Union, Dict, Any

class UpdateInstalledMiniApp(TypedDict, total=False):
    name: Union[str, None]
    identifier: Union[str, None]
    data: Union[Dict[str, Any], None]



from enum import Enum
from typing import TypedDict


class BaseEnum:
    @classmethod
    def get_list(cls):
        return [getattr(cls, attr) for attr in dir(cls) if attr.isupper()]


class ParentType(BaseEnum):
    AUTOMATED_ACTION = "automated_action"
    CAMPAIGN = "campaign"
    ENGAGEMENT_CAMPAIGN = "engagement_campaign"


class ObjectType(BaseEnum):
    INSTALLED_SOURCE = "installed_source"
    INSTALLED_MINI_APP = "installed_mini_app"
    ACTION = "action"


class Organization(TypedDict):
    id: str
    slug: str


class Object(BaseEnum):
    CAMPAIGN = "campaign"
    SEGMENT = "segment"


class BaseStatus(BaseEnum):
    COMPLETED = "completed"


class FileCSVDelimiter(BaseEnum):
    COMMA = ","
    SEMICOLON = ";"
    TAB = "\t"


class Codec:
    UTF8 = "utf-8"
    UTF16 = "utf-16"

    @classmethod
    def get_list(cls):
        return [cls.UTF8, cls.UTF16]


class MessageSenderTypeEnum(str, Enum):
    USER = "user"
    AGENT = "agent"
    SYSTEM = "system"
    AI = "ai"


class PubSubMessageEventNameEnum(str, Enum):
    MESSAGE_CREATED = "Message Created"
    MESSAGE_STATUS_UPDATED = "Message Status Updated"
    CALL_CREATED = "Call Created"


class MessageStatusEnum(str, Enum):
    SENT = "sent"
    FAILED = "failed"
    DELIVERED = "delivered"
    SEEN = "seen"


class EngagementMessageStatus(str, Enum):
    SENT = "sent"
    SENDING_FAILED = "sending_failed"



import csv
import io
from typing import List, Dict, Any, OrderedDict, Union

import pyexcel

from filum_utils.config import config
from filum_utils.enums import Codec, FileCSVDelimiter
from filum_utils.errors import BaseError


class FileService:
    @classmethod
    def get_rows(
        cls,
        file_name: str,
        file_content_bytes: bytes,
        current_index: int,
        limit: int = config.FILE_RECORD_LIMIT
    ) -> List[Dict[str, Any]]:
        rows = cls.get_file_data(file_name, file_content_bytes)
        start_row = current_index
        end_row = start_row + limit

        # prevent getting more rows than total of rows in uploaded file
        if end_row > len(rows):
            end_row = len(rows)

        if start_row == end_row:
            return []

        processed_rows = []
        for i in range(start_row, end_row):
            processed_row = dict(rows[i])
            processed_rows.append(processed_row)

        return processed_rows

    @classmethod
    def get_file_data(cls, file_name: str, file_content_bytes: bytes) -> List[Dict[str, Any]]:
        file_extension = cls._get_file_extension(file_name)
        if file_extension == "csv":
            return cls._get_csv_file_data(file_content_bytes, file_extension)

        if file_extension in ["xlsx", "xls"]:
            return cls._get_excel_file_data(file_content_bytes, file_extension)

        return []

    @classmethod
    def _get_csv_file_data(cls, file_content_bytes: bytes, file_extension: str) -> List[Dict[str, Any]]:
        decoded_file_content = cls._decode_bytes(file_content_bytes)
        delimiter = cls._get_delimiter(decoded_file_content)
        if not delimiter:
            raise BaseError(message="Can not get the delimiter in csv file.")

        records = csv.DictReader(decoded_file_content.splitlines(), skipinitialspace=True, delimiter=delimiter)
        rows = []

        for record in records:
            rows.append(record)

        return rows

    @classmethod
    def _get_excel_file_data(cls, file_content_bytes: bytes, file_extension: str) -> List[OrderedDict]:
        return pyexcel.get_records(
            file_content=io.BytesIO(file_content_bytes),
            file_type=file_extension,
            name_columns_by_row=0
        )

    @staticmethod
    def _get_file_extension(file_name: str):
        string_list = file_name.split(".")
        return string_list[len(string_list) - 1]

    @staticmethod
    def _decode_bytes(default_bytes: bytes) -> str:
        if not default_bytes:
            return ""

        for utf_code in Codec.get_list():
            try:
                return default_bytes.decode(utf_code)
            except UnicodeDecodeError:
                continue

        return ""

    @staticmethod
    def _get_delimiter(default_str) -> Union[str, None]:
        for delimiter in FileCSVDelimiter.get_list():
            # Try parsing the sample using the current delimiter
            try:
                csv.Sniffer().sniff(default_str, delimiters=delimiter)
                return delimiter
            except csv.Error:
                continue

        return None



import os
import sys

from setuptools import setup, find_packages

sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'filum_analytics'))

VERSION = "0.1.39"
DESCRIPTION = "Filum Utils"
LONG_DESCRIPTION = "Filum Utils"

install_requires = [
    "requests==2.31.0",
    "filum-analytics-python==1.1.1",
    "glom==20.11.0",
    "pyexcel==0.7.0",
    "pyexcel-xls==0.7.0",
    "pyexcel-xlsx==0.6.0",
    "openpyxl==3.0.10",
    "setuptools==68.0.0",
    "tenacity==8.2.3",
    "python-dateutil==2.8.2",
    "google-cloud-pubsub==2.13.6"
]

setup(
    name="filum-utils",
    version=VERSION,
    author="Hiep Nguyen",
    author_email="<hnguyen@filum.ai>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=install_requires,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ]
)



from typing import Dict, Any

class ErrorMessage:
    IMPLEMENTATION_ERROR = "Implementation error"
    MISMATCH_LAST_CURRENT_INDEX = "Mismatch last current index"
    MISSING_SEGMENT_ID = "Missing segment id"
    MISSING_FILE = "Missing file"

class BaseError(Exception):
    def __init__(self, message: str, data: Dict[Any, Any] = None):
        self.message = message
        self.data = data
